# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedContent macro, based on the FeedReader macro

    @copyright: 2008, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinRemoteSupport import *
from MoinSupport import parseMacroArguments
import xml.dom.pulldom

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ["time"]

MAX_ENTRIES = 5
ATOM_NS = "http://www.w3.org/2005/Atom"

def text(element):
    nodes = []
    for node in element.childNodes:
        if node.nodeType == node.TEXT_NODE:
            nodes.append(node.nodeValue)
    return "".join(nodes)

def unescape(text):
    return text.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")

def linktext(element, feed_type):
    if feed_type == "rss":
        return text(element)
    else:
        return element.getAttribute("href")

def execute(macro, args):
    request = macro.request
    fmt = macro.formatter
    _ = request.getText

    feed_url = None
    show_content = None
    max_entries = None

    for arg, value in parseMacroArguments(args):
        if arg == "url":
            feed_url = value
        elif arg == "show":
            show_content = value in ("true", "True", "yes")
        elif arg == "limit":
            try:
                max_entries = int(value)
            except ValueError:
                return fmt.text(_("SharedContent: limit must be set to the maximum number of entries to be shown"))

    if not feed_url:
        return fmt.text(_("SharedContent: a feed URL must be specified"))

    show_content = show_content or False
    max_entries = max_entries or MAX_ENTRIES

    # Obtain the resource, using a cached version if appropriate.

    max_cache_age = int(getattr(request.cfg, "moin_share_max_cache_age", "300"))
    data = getCachedResource(request, feed_url, "MoinShare", "wiki", max_cache_age)
    if not data:
        return fmt.text(_("SharedContent: updates could not be retrieved for %s") % feed_url)

    feed = StringIO(data)

    _url, content_type, _encoding, _metadata = getCachedResourceMetadata(feed)

    if content_type not in ("application/atom+xml", "application/rss+xml"):
        return fmt.text(_("SharedContent: updates for %s were not provided in Atom or RSS format") % feed_url)

    try:
        # Parse each node from the feed.

        title = link = content = content_type = None
        channel_title = channel_link = None

        output = []
        append = output.append

        feed_type = None
        in_item = False
        nentries = 0

        events = xml.dom.pulldom.parse(feed)

        if not show_content:
            append(fmt.bullet_list(on=1))

        for event, value in events:

            if event == xml.dom.pulldom.START_ELEMENT:
                tagname = value.localName

                # Detect the feed type and items.

                if tagname == "feed" and value.namespaceURI == ATOM_NS:
                    feed_type = "atom"

                elif tagname == "rss":
                    feed_type = "rss"

                # Detect items.

                elif feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    in_item = True

                elif tagname == "title":
                    events.expandNode(value)
                    if in_item:
                        title = value
                    else:
                        channel_title = value

                elif tagname == "link":
                    events.expandNode(value)
                    if in_item:
                        link = value
                    else:
                        channel_link = value

                elif feed_type == "atom" and tagname == "content":
                    events.expandNode(value)
                    if in_item:
                        content = value
                        content_type = value.getAttribute("type")

            elif event == xml.dom.pulldom.END_ELEMENT:
                tagname = value.localName

                if feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    in_item = False

                    # Emit content where appropriate.
                    # NOTE: HTML should be sanitised.

                    if show_content:
                        if content and content_type == "html":
                            append(fmt.rawHTML(unescape(text(content))))

                    # Or emit title and link information for items.

                    elif title and link and nentries < max_entries:
                        link_text = linktext(link, feed_type)

                        append(fmt.listitem(on=1))
                        append(fmt.url(on=1, href=link_text))
                        append(fmt.icon('www'))
                        append(fmt.text(" " + text(title)))
                        append(fmt.url(on=0))
                        append(fmt.listitem(on=0))

                    title = link = content = content_type = None
                    nentries += 1

        if not show_content:
            append(fmt.bullet_list(on=0))

        if channel_title and channel_link:
            channel_link_text = linktext(channel_link, feed_type)

            append(fmt.paragraph(on=1))
            append(fmt.url(on=1, href=channel_link_text))
            append(fmt.text(text(channel_title)))
            append(fmt.url(on=0))
            append(fmt.text(" "))
            append(fmt.url(on=1, href=feed_url))
            append(fmt.icon('rss'))
            append(fmt.url(on=0))
            append(fmt.paragraph(on=0))

    finally:
        feed.close()

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
